package org.codehaus.plexus.archiver.bzip2;

import javax.inject.Named;

/**
 * Alias of {@link PlexusIoBzip2ResourceCollection}.
 */
@Named( "bz2" )
public class PlexusIoBz2ResourceCollection
    extends PlexusIoBzip2ResourceCollection
{
}
