package org.codehaus.plexus.archiver.tar;

import javax.inject.Named;

import java.io.File;

/**
 * Alias for {@link PlexusIoTarBZip2FileResourceCollection}
 */
@Named( "tbz2" )
public class PlexusIoTBZ2FileResourceCollection
    extends PlexusIoTarBZip2FileResourceCollection
{
}
