package org.codehaus.plexus.archiver.tar;

import javax.inject.Named;

import java.io.File;

/**
 * Alias for {@link PlexusIoTarGZipFileResourceCollection}
 */
@Named( "tgz" )
public class PlexusIoTGZFileResourceCollection
    extends PlexusIoTarGZipFileResourceCollection
{
}
