package org.codehaus.plexus.archiver;

public class ArchiveFilterException
    extends Exception
{

    public ArchiveFilterException( String message, Throwable cause )
    {
        super( message, cause );
    }

    public ArchiveFilterException( String message )
    {
        super( message );
    }

}
