/**
 *
 * Copyright 2004 The Apache Software Foundation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.codehaus.plexus.archiver.esb;

import javax.inject.Named;

import java.io.File;

import org.codehaus.plexus.archiver.zip.ZipUnArchiver;

/**
 * Alias for {@link ZipUnArchiver}.
 */
@Named( "esb" )
public class EsbUnArchiver
        extends ZipUnArchiver
{

    public EsbUnArchiver()
    {
    }

    public EsbUnArchiver( File sourceFile )
    {
        super( sourceFile );
    }

}
