package org.codehaus.plexus.archiver.gzip;

import javax.inject.Named;

/**
 * Alias for {@link PlexusIoGzipResourceCollection}
 */
@Named( "gz" )
public class PlexusIoGzResourceCollection
        extends PlexusIoGzipResourceCollection
{
}
